@@ -101,6 +101,7 @@ NS_INTERFACE_MAP_BEGIN(EmbedEventListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMMouseListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMUIListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFocusListener)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMContextMenuListener)
 NS_INTERFACE_MAP_END
 
 nsresult
@@ -204,14 +205,27 @@ EmbedEventListener::HandleLink(nsIDOMNode* node)
 
   // XXX This does not handle |BLAH ICON POWER"
   if (name.LowerCaseEqualsLiteral("alternate") &&
-      type.LowerCaseEqualsLiteral("application/rss+xml")) {
+      (type.LowerCaseEqualsLiteral("application/rss+xml") ||
+       type.LowerCaseEqualsLiteral("application/atom+xml"))) {
 
     NS_ConvertUTF16toUTF8 narrowTitle(title);
 
-    g_signal_emit(GTK_OBJECT(mOwner->mOwningWidget),
-                  moz_embed_signals[RSS_REQUEST], 0,
-                  (gchar *)url.get(),
-                  narrowTitle.get());
+    // Ignore RSS links from inside iframes
+    nsCOMPtr<nsIDOMWindow> win;
+    nsCOMPtr<nsIDOMWindow> parent_win;
+    rv = GetDOMWindowByNode (node, getter_AddRefs (win));
+
+    if (NS_SUCCEEDED (rv) && win)
+      rv = win->GetParent (getter_AddRefs (parent_win));
+
+    // Note: we allow the signal to be emitted "just in case" if getting parent 
+    // window failed for some reason.
+    if (NS_FAILED (rv) || parent_win == win)
+      g_signal_emit(GTK_OBJECT(mOwner->mOwningWidget),
+                    moz_embed_signals[RSS_REQUEST], 0,
+                    (gchar *)url.get(),
+                    narrowTitle.get());
+
     mXSLTransformed = PR_FALSE;
   }
   return NS_OK;
@@ -598,6 +612,24 @@ EmbedEventListener::Blur(nsIDOMEvent* aEvent)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+EmbedEventListener::ContextMenu(nsIDOMEvent* aDOMEvent)
+{
+  nsString eventType;
+  aDOMEvent->GetType(eventType);
+
+  if (eventType.EqualsLiteral("contextmenu")) {
+    gint return_val = FALSE;
+    g_signal_emit(GTK_OBJECT(mOwner->mOwningWidget),
+                  moz_embed_signals[DOM_CONTEXT_MENU], 0, (void *)aDOMEvent, &return_val);
+    if (return_val) {
+      aDOMEvent->StopPropagation();
+      aDOMEvent->PreventDefault();
+    }
+  }
+  return NS_OK;  
+}
+
 nsresult
 EmbedEventListener::NewURI(nsIURI **result,
                             const char *spec)
