@@ -1108,6 +1108,13 @@ EmbedPrivate::AttachListeners(void)
     return;
   }
 
+  rv = mEventTarget->AddEventListenerByIID(eventListener,
+                                             NS_GET_IID(nsIDOMContextMenuListener));
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Failed to add context menu listener\n");
+    return;
+  }
+  
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(mEventTarget));
   if (!target) {
     return;
